-- Designed by Toshio Iwata at DIGITALFILTER.COM 2013/03/19

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LINEOUT_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
--		SoftReset_N	: IN std_logic;
		ViewMode : in std_logic_vector(3 downto 0);
--		BigLineN : in std_logic_vector(8 downto 0);
--		RD_ADDR_LOAD : in std_logic;
--		RD_ADDR_ENB : in std_logic;
--		WR_ADDR_CLR : in std_logic;
		PIXSTB: in std_logic;
		PIXWR : in std_logic;
		PIXRD : in std_logic;
  RD_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : out std_logic_vector(1 downto 0);
  WR_BUF_N : out std_logic;
  RD_BUF_ADDR : out std_logic_vector(8 downto 0);
  SsdLineCount : out std_logic_vector(8 downto 0);
  SsdLineCount2 : out std_logic_vector(8 downto 0);
  SsdPixCount : out std_logic_vector(8 downto 0);
  SsdPixCount2 : out std_logic_vector(8 downto 0);
  CO9 : out std_logic_vector(3 downto 0);
  CO64 : out std_logic_vector(5 downto 0);
  CO64_dly2 : out std_logic_vector(5 downto 0);
  ssd_RGB : out std_logic_vector(15 downto 0);
  PIX2LOGIC : in std_logic_vector(15 downto 0);
LBRDY : out std_logic;
CamHsync_edge : in std_logic;
VgaHsync_edge : in std_logic;
DISPJUDGE : in std_logic;
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		CamLineCount : in std_logic_vector(8 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
		winp3: in std_logic );
END LINEOUT_CTRL;

ARCHITECTURE structure OF LINEOUT_CTRL IS

component SRAM_512x16
   Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(8 downto 0);
          RDADDR : In std_logic_vector(8 downto 0);
          WRDATA : In std_logic_vector(15 downto 0);
          RDDATA : Out std_logic_vector(15 downto 0) );
end component;

 signal OB_WR_ADDR : std_logic_vector(8 downto 0);
 signal OB_RD_ADDR : std_logic_vector(8 downto 0);
 signal OB_WR_DATA : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_0 : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_1 : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_2 : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_3 : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_3_padding : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_0_padding : std_logic_vector(15 downto 0);
 signal OB_WR_N_0 : std_logic;
 signal OB_WR_N_1 : std_logic;
 signal OB_WR_N_2 : std_logic;
 signal OB_WR_N_3 : std_logic;
 signal OB_WR_DATA_sig : std_logic_vector(15 downto 0);
 signal OB_WR_N_0_sig : std_logic;
 signal OB_WR_N_1_sig : std_logic;
 signal OB_WR_N_2_sig : std_logic;
 signal OB_WR_N_3_sig : std_logic;
signal ODDLINE, oddline_dly1, oddline_dly2, oddline_dly3, oddline_edge : std_logic;
 signal ls2b : std_logic_vector(1 downto 0);
 signal LB_ADDR : std_logic_vector(8 downto 0);
signal PIXRD_edge, PIXRD_dly1, PIXRD_dly2 : std_logic;
signal PIXWR_edge, PIXWR_dly1, PIXWR_dly2 : std_logic;
signal WaitLine : std_logic_vector(2 downto 0);
signal  pixstb_count : std_logic_vector(9 downto 0);
signal  pixstb_count_plus : std_logic_vector(9 downto 0);
signal LBRDY_sig : std_logic;
	signal ViewMode_dly1, ViewMode_dly2 : std_logic_vector(3 downto 0);
  signal ViewMode5_edge : std_logic;
  signal ViewMode6_edge : std_logic;
 signal count4wraddr : std_logic_vector(9 downto 0);
signal  wr_r_6bit, wr_g_6bit, wr_b_6bit : std_logic_vector(5 downto 0);
signal wr_y, wr_y_tmp : std_logic_vector(7 downto 0);
signal  vga_r_6bit, vga_g_6bit, vga_b_6bit : std_logic_vector(5 downto 0);
signal SsdStart : std_logic;
signal count64 : std_logic_vector(5 downto 0); -- counts input channels(+1)
signal count64_clr : std_logic;
signal count64_dly1, count64_dly2 : std_logic_vector(5 downto 0);
signal count3 : std_logic_vector(8 downto 0); -- counts horizontal 3 pixs
signal count9 : std_logic_vector(3 downto 0); -- counts 3x3 pixs
signal count9_clr : std_logic;
signal count9_dly1, count9_dly2 : std_logic_vector(3 downto 0);
signal count300 : std_logic_vector(8 downto 0); -- counts 300 areas
signal count300_dly1, count300_dly2 : std_logic_vector(8 downto 0);
signal count300_clr : std_logic;
signal count300_64_dly1, count300_64_dly2, count300_64_dly3, count300_64_dly4 : std_logic_vector(8 downto 0);
signal countV : std_logic_vector(8 downto 0);
signal countV_clr : std_logic;
signal rdbufsel : std_logic_vector(1 downto 0);
signal rdbufsel_dly1, rdbufsel_dly2 : std_logic_vector(1 downto 0);
signal loadval : std_logic_vector(1 downto 0);
signal WR_ADDR_CLR_dly1, WR_ADDR_CLR_dly2, WR_ADDR_CLR_edge : std_logic;
signal RD_ADDR_ENB_dly1, RD_ADDR_ENB_dly2, RD_ADDR_ENB_edge : std_logic;
signal WRN2DL_sig, DISPJUDGE_latch : std_logic;
signal countV_dly1, countV_dly2, countV_dly3, countV_dly4 : std_logic_vector(8 downto 0);
signal WR_N_X : std_logic_vector(1 downto 0);

BEGIN

  wr_r_6bit <= OB_WR_DATA_sig(15 downto 11) & '0';
  wr_g_6bit <= OB_WR_DATA_sig(10 downto 5);
  wr_b_6bit <= OB_WR_DATA_sig(4 downto 0) & '0';

  wr_y_tmp <= ("00" & wr_r_6bit) + ("00" & wr_g_6bit) + ("00" & wr_b_6bit); 
  wr_y <= "11110000" when wr_y_tmp >= 64 else "00000000";

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     oddline_dly3 <= '0';
     oddline_dly2 <= '0';
     oddline_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     oddline_dly3 <= oddline_dly2;
     oddline_dly2 <= oddline_dly1;
     oddline_dly1 <= ODDLINE;
   end if;  
end process;

  ODDLINE <= VgaLineCount(1);
  oddline_edge <= '1' when (oddline_dly1 = '0' and oddline_dly2 = '1') else '0';

  WR_BUF_N <= not count64_clr;
  WR_BUF_SEL <= countV_dly1(1 downto 0);
  WR_BUF_SEL2 <= countV_dly4(2 downto 1); -- because of pooling

-------------------------------------------------------------- 

  ssd_RGB <= OB_RD_DATA_0_padding when rdbufsel_dly2 = 0 else 
 		OB_RD_DATA_1 when rdbufsel_dly2 = 1 else 
 		OB_RD_DATA_2 when rdbufsel_dly2 = 2 else 
 		OB_RD_DATA_3_padding;

OB_RD_DATA_3_padding <= "1000010000010000" when countV = 2 else OB_RD_DATA_3;

OB_RD_DATA_0_padding <= "1000010000010000" when countV = 301 else OB_RD_DATA_0;

------------------------------------------------------------------- 
  WR_N_X <= countV(1 downto 0);

  OB_WR_N_0 <= '0' when (WR_N_X = 0 and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     ) else '1';
                       
  OB_WR_N_1 <= '0' when (WR_N_X = 1 and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     ) else '1';
  
  OB_WR_N_2 <= '0' when (WR_N_X = 2 and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     ) else '1';
  
  OB_WR_N_3 <= '0' when (WR_N_X = 3 and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     ) else '1';
  
  OB_WR_DATA_sig <= PIX2LOGIC(15 downto 0) when ViewMode = "0110" else
                   latch0(15 downto 0) when winp0 = '1' else
                   latch1(15 downto 0) when winp1 = '1' else
                   latch2(15 downto 0) when winp2 = '1' else latch3(15 downto 0);
  
  WRN2DL_sig <= '0' when (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     else '1';
                       
-------------------------------------------------------------------
  ls2b <= "00" when winp0 = '1' else
          "01" when winp1 = '1' else
          "10" when winp2 = '1' else
          "11";

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
	if( RST_N = '0' ) then
		LB_ADDR <= (others => '0');
	elsif( CLK100M'event and CLK100M = '1' ) then
		LB_ADDR <= count300 + count3;
	end if;  
end process;
          
-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     countV <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(SsdStart = '1' ) then
         countV <= (others => '0');
       elsif(count300_clr = '1' and countV /= 511) then 
         countV <= countV + 1;
       end if;
   end if;
end process;

--countV_clr <= '1' when countV = 299 and count300 = 299 and count64 = 63 and count9 = 8 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     countV_dly4 <= (others => '0');
     countV_dly3 <= (others => '0');
     countV_dly2 <= (others => '0');
     countV_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(count64_clr = '1') then
         countV_dly4 <= countV_dly3; -- for writing Relu-out
         countV_dly3 <= countV_dly2; -- for writing Relu-out
         countV_dly2 <= countV_dly1; -- for writing Relu-out
         countV_dly1 <= countV; -- for writing Relu-out
       end if;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     count300 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(count300_clr = '1' or SsdStart = '1' ) then
         count300 <= (others => '0');
       elsif(count64_clr = '1') then
         count300 <= count300 + 1;
       end if;
   end if;
end process;

count300_clr <= '1' when count300 = 299 and count64 = 63 and count9 = 8 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then -- or SoftReset_N = '0' ) then
     count64 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(count64_clr = '1' or SsdStart = '1' ) then
       count64 <= (others => '0');
     elsif(count9_clr = '1') then
       count64 <= count64 + 1;
     end if;
  end if;
end process;

count64_clr <= '1' when count64 = 63 and count9 = 8 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then -- or SoftReset_N = '0' ) then
     count9 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(count9_clr = '1' or  SsdStart = '1' ) then 
       count9 <= (others => '0');
     else
       count9 <= count9 + 1;
     end if;
  end if;
end process;

  count9_clr <= '1' when count9 = 8 else '0';

  count3 <= "000000000" when count9 = 0 or count9 = 3 or  count9 = 6 else 
		"000000001" when count9 = 1 or count9 = 4 or  count9 = 7 else 
		"000000010";

  rdbufsel <= WR_N_X - 3 when count9 = 0 or count9 = 1 or count9 = 2 else
		WR_N_X - 2 when count9 = 3 or count9 = 4 or count9 = 5 else
		WR_N_X - 1;

  RD_BUF_SEL <= rdbufsel_dly2;

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    DISPJUDGE_latch <= '0';
  elsif( CLK'event and CLK = '1' ) then
    if(VgaHsync_edge = '1') then
      DISPJUDGE_latch <= DISPJUDGE;
    end if;
  end if;
end process;

  SsdStart <= '1' when VgaHsync_edge = '1' and DISPJUDGE_latch = '0' else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     rdbufsel_dly2 <= (others => '0');
     rdbufsel_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     rdbufsel_dly2 <= rdbufsel_dly1;
     rdbufsel_dly1 <= rdbufsel;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count9_dly2 <= (others => '0');
     count9_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count9_dly2 <= count9_dly1;
     count9_dly1 <= count9;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count64_dly2 <= (others => '0');
     count64_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count64_dly2 <= count64_dly1;
     count64_dly1 <= count64;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count300_dly2 <= (others => '0');
     count300_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count300_dly2 <= count300_dly1;
     count300_dly1 <= count300;
   end if;
end process;

  SsdLineCount <= countV;
  SsdLineCount2 <= countV_dly4;
  SsdPixCount <= "100101100" when count300_dly1 = 0 else count300_dly1; -- write address 1 to 300 for padding
  SsdPixCount2 <= count300_64_dly4; -- for layer2 this would be better (not tricky)
  CO9 <= count9;
  CO64 <= count64;
  CO64_dly2 <= count64_dly2;

process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count300_64_dly4 <= (others => '0');
     count300_64_dly3 <= (others => '0');
     count300_64_dly2 <= (others => '0');
     count300_64_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
      if(count64_clr = '1') then
         count300_64_dly4 <= count300_64_dly3;
         count300_64_dly3 <= count300_64_dly2;
         count300_64_dly2 <= count300_64_dly1;
         count300_64_dly1 <= count300;
      end if;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count4wraddr <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(VgaHsync_edge = '1') then 
       count4wraddr <= (others => '0');
     elsif(WRN2DL_sig = '0' and count4wraddr /= 607) then -- 1023) then
       count4wraddr <= count4wraddr + 1;
     end if;
  end if;
end process;

-------------------------------------------------------------- 
  OB_WR_ADDR <= count4wraddr(9 downto 1) + 1;  -- plus 1 for padding address
  OB_RD_ADDR <= LB_ADDR;
  RD_BUF_ADDR <= LB_ADDR;

-------------------------------------------------------------- 
--process( RST_N, CLK )  
--begin 
--   if( RST_N = '0' ) then
--     OB_WR_N_0 <= '0';
--     OB_WR_N_1 <= '0';
--     OB_WR_N_2 <= '0';
--     OB_WR_N_3 <= '0';
--     WRN2DL <= '0';
--   elsif( CLK'event and CLK = '1' ) then
--     OB_WR_N_0 <= OB_WR_N_0_sig;
--     OB_WR_N_1 <= OB_WR_N_1_sig;
--     OB_WR_N_2 <= OB_WR_N_2_sig;
--     OB_WR_N_3 <= OB_WR_N_3_sig;
--     WRN2DL <= WRN2DL_sig;
--  end if;
--end process;

OB_WR_DATA <= OB_WR_DATA_sig;

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0') then
      PIXRD_dly2 <= '0';
      PIXRD_dly1 <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PIXRD_dly2 <= PIXRD_dly1;
    PIXRD_dly1 <= PIXRD;
  end if;
end process;

  PIXRD_edge <= '1' when PIXRD_dly1 = '0' and PIXRD_dly2 = '1' else '0';
  
-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0') then
      PIXWR_dly2 <= '0';
      PIXWR_dly1 <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PIXWR_dly2 <= PIXWR_dly1;
    PIXWR_dly1 <= PIXWR;
  end if;
end process;

  PIXWR_edge <= '1' when PIXWR_dly1 = '0' and PIXWR_dly2 = '1' else '0';

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    WaitLine <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(PIXWR_edge = '1' or PIXRD_edge = '1') then
      WaitLine <= (others => '0');
    elsif(CamHsync_edge = '1') then
      if(WaitLine /= 3) then -- 2012/11/15
        WaitLine <= WaitLine + 1;
      end if;
    end if;
  end if;
end process;

  LBRDY_sig <= '1' when not(WaitLine = 1 or WaitLine = 2) and (ViewMode = "0101" or ViewMode = "0110") else '0'; -- 2012/11/15
  LBRDY <= LBRDY_sig;
  
-------------------------------------------------------------- 
process( RST_N, PIXSTB, PIXRD_edge, PIXWR_edge )  
begin 
   if( RST_N = '0' or PIXRD_edge = '1'  or PIXWR_edge = '1' ) then
     pixstb_count <= (others => '0');
   elsif( PIXSTB'event and PIXSTB = '1' ) then
     pixstb_count <= pixstb_count + 1;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     pixstb_count_plus <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     pixstb_count_plus <= pixstb_count + 96;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     ViewMode_dly2 <= (others => '0');
     ViewMode_dly1 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     ViewMode_dly2 <= ViewMode_dly1;
     ViewMode_dly1 <= ViewMode;
   end if;  
end process;
   
-------------------------------------------------------------- 
  ViewMode5_edge <= '1' when ViewMode_dly1 = "0101" and ViewMode_dly2 /= "0101" else '0';
  ViewMode6_edge <= '1' when ViewMode_dly1 = "0110" and ViewMode_dly2 /= "0110" else '0';

END structure;
