-- The layer2 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer2 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO64 : in std_logic_vector(5 downto 0);
  CO64_dly2 : in std_logic_vector(5 downto 0);
  X_RD_DATA : in std_logic_vector(319 downto 0); 
  ReluAll : out std_logic_vector(319 downto 0); 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_layer2;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer2 is

component dotpro_W2
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(1 downto 0);
  CO9is2 : in std_logic;
  CO64is63 : in std_logic;
  MATMULOUT : Out std_logic_vector(12 downto 0) );  
end component;

component ROM_W2
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(9 downto 0);
          DATA : Out std_logic_vector(127 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_00, mac_01, mac_02, mac_03, mac_04, mac_05, mac_06, mac_07, mac_08, mac_09 : std_logic_vector(12 downto 0);
signal mac_10, mac_11, mac_12, mac_13, mac_14, mac_15, mac_16, mac_17, mac_18, mac_19 : std_logic_vector(12 downto 0);
signal mac_20, mac_21, mac_22, mac_23, mac_24, mac_25, mac_26, mac_27, mac_28, mac_29 : std_logic_vector(12 downto 0);
signal mac_30, mac_31, mac_32, mac_33, mac_34, mac_35, mac_36, mac_37, mac_38, mac_39 : std_logic_vector(12 downto 0);
signal mac_40, mac_41, mac_42, mac_43, mac_44, mac_45, mac_46, mac_47, mac_48, mac_49 : std_logic_vector(12 downto 0);
signal mac_50, mac_51, mac_52, mac_53, mac_54, mac_55, mac_56, mac_57, mac_58, mac_59 : std_logic_vector(12 downto 0);
signal mac_60, mac_61, mac_62, mac_63 : std_logic_vector(12 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09 : std_logic_vector(6 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15, truncate_16, truncate_17, truncate_18, truncate_19 : std_logic_vector(6 downto 0);
signal truncate_20, truncate_21, truncate_22, truncate_23, truncate_24, truncate_25, truncate_26, truncate_27, truncate_28, truncate_29 : std_logic_vector(6 downto 0);
signal truncate_30, truncate_31, truncate_32, truncate_33, truncate_34, truncate_35, truncate_36, truncate_37, truncate_38, truncate_39 : std_logic_vector(6 downto 0);
signal truncate_40, truncate_41, truncate_42, truncate_43, truncate_44, truncate_45, truncate_46, truncate_47, truncate_48, truncate_49 : std_logic_vector(6 downto 0);
signal truncate_50, truncate_51, truncate_52, truncate_53, truncate_54, truncate_55, truncate_56, truncate_57, truncate_58, truncate_59 : std_logic_vector(6 downto 0);
signal truncate_60, truncate_61, truncate_62, truncate_63 : std_logic_vector(6 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09 : std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15, Relu_16, Relu_17, Relu_18, Relu_19 : std_logic_vector(4 downto 0);
signal Relu_20, Relu_21, Relu_22, Relu_23, Relu_24, Relu_25, Relu_26, Relu_27, Relu_28, Relu_29 : std_logic_vector(4 downto 0);
signal Relu_30, Relu_31, Relu_32, Relu_33, Relu_34, Relu_35, Relu_36, Relu_37, Relu_38, Relu_39 : std_logic_vector(4 downto 0);
signal Relu_40, Relu_41, Relu_42, Relu_43, Relu_44, Relu_45, Relu_46, Relu_47, Relu_48, Relu_49 : std_logic_vector(4 downto 0);
signal Relu_50, Relu_51, Relu_52, Relu_53, Relu_54, Relu_55, Relu_56, Relu_57, Relu_58, Relu_59 : std_logic_vector(4 downto 0);
signal Relu_60, Relu_61, Relu_62, Relu_63 : std_logic_vector(4 downto 0);
signal PixRomData : std_logic_vector(4 downto 0);
signal W2Addr, W2Addr_prev : std_logic_vector(9 downto 0);
signal W2Addr_L : std_logic_vector(3 downto 0);
signal W2Addr_H : std_logic_vector(9 downto 0);
signal W2Data : std_logic_vector(127 downto 0);
signal coeff_00, coeff_01, coeff_02, coeff_03, coeff_04, coeff_05, coeff_06, coeff_07, coeff_08, coeff_09: std_logic_vector(1 downto 0);
signal coeff_10, coeff_11, coeff_12, coeff_13, coeff_14, coeff_15, coeff_16, coeff_17, coeff_18, coeff_19: std_logic_vector(1 downto 0);
signal coeff_20, coeff_21, coeff_22, coeff_23, coeff_24, coeff_25, coeff_26, coeff_27, coeff_28, coeff_29: std_logic_vector(1 downto 0);
signal coeff_30, coeff_31, coeff_32, coeff_33, coeff_34, coeff_35, coeff_36, coeff_37, coeff_38, coeff_39: std_logic_vector(1 downto 0);
signal coeff_40, coeff_41, coeff_42, coeff_43, coeff_44, coeff_45, coeff_46, coeff_47, coeff_48, coeff_49: std_logic_vector(1 downto 0);
signal coeff_50, coeff_51, coeff_52, coeff_53, coeff_54, coeff_55, coeff_56, coeff_57, coeff_58, coeff_59: std_logic_vector(1 downto 0);
signal coeff_60, coeff_61, coeff_62, coeff_63: std_logic_vector(1 downto 0);
signal CO9is2, CO64is63: std_logic;

begin
---------------------------------------------------------
  PixRomData <= 
		X_RD_DATA(	319	downto	315	) when CO64_dly2 = "000000" else 
		X_RD_DATA(	314	downto	310	) when CO64_dly2 = "000001" else 
		X_RD_DATA(	309	downto	305	) when CO64_dly2 = "000010" else 
		X_RD_DATA(	304	downto	300	) when CO64_dly2 = "000011" else 
		X_RD_DATA(	299	downto	295	) when CO64_dly2 = "000100" else 
		X_RD_DATA(	294	downto	290	) when CO64_dly2 = "000101" else 
		X_RD_DATA(	289	downto	285	) when CO64_dly2 = "000110" else 
		X_RD_DATA(	284	downto	280	) when CO64_dly2 = "000111" else 
		X_RD_DATA(	279	downto	275	) when CO64_dly2 = "001000" else 
		X_RD_DATA(	274	downto	270	) when CO64_dly2 = "001001" else 
		X_RD_DATA(	269	downto	265	) when CO64_dly2 = "001010" else 
		X_RD_DATA(	264	downto	260	) when CO64_dly2 = "001011" else 
		X_RD_DATA(	259	downto	255	) when CO64_dly2 = "001100" else 
		X_RD_DATA(	254	downto	250	) when CO64_dly2 = "001101" else 
		X_RD_DATA(	249	downto	245	) when CO64_dly2 = "001110" else 
		X_RD_DATA(	244	downto	240	) when CO64_dly2 = "001111" else 
		X_RD_DATA(	239	downto	235	) when CO64_dly2 = "010000" else 
		X_RD_DATA(	234	downto	230	) when CO64_dly2 = "010001" else 
		X_RD_DATA(	229	downto	225	) when CO64_dly2 = "010010" else 
		X_RD_DATA(	224	downto	220	) when CO64_dly2 = "010011" else 
		X_RD_DATA(	219	downto	215	) when CO64_dly2 = "010100" else 
		X_RD_DATA(	214	downto	210	) when CO64_dly2 = "010101" else 
		X_RD_DATA(	209	downto	205	) when CO64_dly2 = "010110" else 
		X_RD_DATA(	204	downto	200	) when CO64_dly2 = "010111" else 
		X_RD_DATA(	199	downto	195	) when CO64_dly2 = "011000" else 
		X_RD_DATA(	194	downto	190	) when CO64_dly2 = "011001" else 
		X_RD_DATA(	189	downto	185	) when CO64_dly2 = "011010" else 
		X_RD_DATA(	184	downto	180	) when CO64_dly2 = "011011" else 
		X_RD_DATA(	179	downto	175	) when CO64_dly2 = "011100" else 
		X_RD_DATA(	174	downto	170	) when CO64_dly2 = "011101" else 
		X_RD_DATA(	169	downto	165	) when CO64_dly2 = "011110" else 
		X_RD_DATA(	164	downto	160	) when CO64_dly2 = "011111" else 
		X_RD_DATA(	159	downto	155	) when CO64_dly2 = "100000" else 
		X_RD_DATA(	154	downto	150	) when CO64_dly2 = "100001" else 
		X_RD_DATA(	149	downto	145	) when CO64_dly2 = "100010" else 
		X_RD_DATA(	144	downto	140	) when CO64_dly2 = "100011" else 
		X_RD_DATA(	139	downto	135	) when CO64_dly2 = "100100" else 
		X_RD_DATA(	134	downto	130	) when CO64_dly2 = "100101" else 
		X_RD_DATA(	129	downto	125	) when CO64_dly2 = "100110" else 
		X_RD_DATA(	124	downto	120	) when CO64_dly2 = "100111" else 
		X_RD_DATA(	119	downto	115	) when CO64_dly2 = "101000" else 
		X_RD_DATA(	114	downto	110	) when CO64_dly2 = "101001" else 
		X_RD_DATA(	109	downto	105	) when CO64_dly2 = "101010" else 
		X_RD_DATA(	104	downto	100	) when CO64_dly2 = "101011" else 
		X_RD_DATA(	99	downto	95	) when CO64_dly2 = "101100" else 
		X_RD_DATA(	94	downto	90	) when CO64_dly2 = "101101" else 
		X_RD_DATA(	89	downto	85	) when CO64_dly2 = "101110" else 
		X_RD_DATA(	84	downto	80	) when CO64_dly2 = "101111" else 
		X_RD_DATA(	79	downto	75	) when CO64_dly2 = "110000" else 
		X_RD_DATA(	74	downto	70	) when CO64_dly2 = "110001" else 
		X_RD_DATA(	69	downto	65	) when CO64_dly2 = "110010" else 
		X_RD_DATA(	64	downto	60	) when CO64_dly2 = "110011" else 
		X_RD_DATA(	59	downto	55	) when CO64_dly2 = "110100" else 
		X_RD_DATA(	54	downto	50	) when CO64_dly2 = "110101" else 
		X_RD_DATA(	49	downto	45	) when CO64_dly2 = "110110" else 
		X_RD_DATA(	44	downto	40	) when CO64_dly2 = "110111" else 
		X_RD_DATA(	39	downto	35	) when CO64_dly2 = "111000" else 
		X_RD_DATA(	34	downto	30	) when CO64_dly2 = "111001" else 
		X_RD_DATA(	29	downto	25	) when CO64_dly2 = "111010" else 
		X_RD_DATA(	24	downto	20	) when CO64_dly2 = "111011" else 
		X_RD_DATA(	19	downto	15	) when CO64_dly2 = "111100" else 
		X_RD_DATA(	14	downto	10	) when CO64_dly2 = "111101" else 
		X_RD_DATA(	9	downto	5	) when CO64_dly2 = "111110" else 
		X_RD_DATA(	4	downto	0	) ; 

W2Addr_L <= CO9;
W2Addr_H <= ("0" & CO64 & "000") + ("0000" & CO64);
W2Addr_prev <= W2Addr_H + ("000000" & W2Addr_L);

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		W2Addr <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(W2Addr_prev >= -448 and W2Addr_prev <= -1) then -- note that this is Signed
			W2Addr <= (others => '0');
		else
			W2Addr <= W2Addr_prev;
		end if;
	end if;  
end process;

---------------------------------------------------------
CO9is2 <= '1' when CO9 = 1 else '0'; -- from 2 to 1 because already delayed in LineOut_Ctrl.vhd

CO64is63 <= '1' when CO64 = 0 else '0';

ROM_W2_inst : ROM_W2 port map (
  CLK => CLK,  
  ADDR => W2Addr, 
  DATA => W2Data);

---------------------------------------------------------
coeff_00 <= W2Data( 127 downto 126 );
coeff_01 <= W2Data( 125 downto 124 );
coeff_02 <= W2Data( 123 downto 122 );
coeff_03 <= W2Data( 121 downto 120 );
coeff_04 <= W2Data( 119 downto 118 );
coeff_05 <= W2Data( 117 downto 116 );
coeff_06 <= W2Data( 115 downto 114 );
coeff_07 <= W2Data( 113 downto 112 );
coeff_08 <= W2Data( 111 downto 110 );
coeff_09 <= W2Data( 109 downto 108 );
coeff_10 <= W2Data( 107 downto 106 );
coeff_11 <= W2Data( 105 downto 104 );
coeff_12 <= W2Data( 103 downto 102 );
coeff_13 <= W2Data( 101 downto 100 );
coeff_14 <= W2Data( 99 downto 98 );
coeff_15 <= W2Data( 97 downto 96 );
coeff_16 <= W2Data( 95 downto 94 );
coeff_17 <= W2Data( 93 downto 92 );
coeff_18 <= W2Data( 91 downto 90 );
coeff_19 <= W2Data( 89 downto 88 );
coeff_20 <= W2Data( 87 downto 86 );
coeff_21 <= W2Data( 85 downto 84 );
coeff_22 <= W2Data( 83 downto 82 );
coeff_23 <= W2Data( 81 downto 80 );
coeff_24 <= W2Data( 79 downto 78 );
coeff_25 <= W2Data( 77 downto 76 );
coeff_26 <= W2Data( 75 downto 74 );
coeff_27 <= W2Data( 73 downto 72 );
coeff_28 <= W2Data( 71 downto 70 );
coeff_29 <= W2Data( 69 downto 68 );
coeff_30 <= W2Data( 67 downto 66 );
coeff_31 <= W2Data( 65 downto 64 );
coeff_32 <= W2Data( 63 downto 62 );
coeff_33 <= W2Data( 61 downto 60 );
coeff_34 <= W2Data( 59 downto 58 );
coeff_35 <= W2Data( 57 downto 56 );
coeff_36 <= W2Data( 55 downto 54 );
coeff_37 <= W2Data( 53 downto 52 );
coeff_38 <= W2Data( 51 downto 50 );
coeff_39 <= W2Data( 49 downto 48 );
coeff_40 <= W2Data( 47 downto 46 );
coeff_41 <= W2Data( 45 downto 44 );
coeff_42 <= W2Data( 43 downto 42 );
coeff_43 <= W2Data( 41 downto 40 );
coeff_44 <= W2Data( 39 downto 38 );
coeff_45 <= W2Data( 37 downto 36 );
coeff_46 <= W2Data( 35 downto 34 );
coeff_47 <= W2Data( 33 downto 32 );
coeff_48 <= W2Data( 31 downto 30 );
coeff_49 <= W2Data( 29 downto 28 );
coeff_50 <= W2Data( 27 downto 26 );
coeff_51 <= W2Data( 25 downto 24 );
coeff_52 <= W2Data( 23 downto 22 );
coeff_53 <= W2Data( 21 downto 20 );
coeff_54 <= W2Data( 19 downto 18 );
coeff_55 <= W2Data( 17 downto 16 );
coeff_56 <= W2Data( 15 downto 14 );
coeff_57 <= W2Data( 13 downto 12 );
coeff_58 <= W2Data( 11 downto 10 );
coeff_59 <= W2Data( 9 downto 8 );
coeff_60 <= W2Data( 7 downto 6 );
coeff_61 <= W2Data( 5 downto 4 );
coeff_62 <= W2Data( 3 downto 2 );
coeff_63 <= W2Data( 1 downto 0 );

---------------------------------------------------------
dotpro_W2_00 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_00,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_00 );

dotpro_W2_01 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_01,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_01 );

dotpro_W2_02 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_02,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_02 );

dotpro_W2_03 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_03,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_03 );

dotpro_W2_04 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_04,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_04 );

dotpro_W2_05 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_05,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_05 );

dotpro_W2_06 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_06,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_06 );

dotpro_W2_07 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_07,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_07 );

dotpro_W2_08 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_08,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_08 );

dotpro_W2_09 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_09,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_09 );

dotpro_W2_10 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_10,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_10 );

dotpro_W2_11 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_11,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_11 );

dotpro_W2_12 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_12,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_12 );

dotpro_W2_13 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_13,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_13 );

dotpro_W2_14 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_14,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_14 );

dotpro_W2_15 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_15,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_15 );

dotpro_W2_16 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_16,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_16 );

dotpro_W2_17 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_17,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_17 );

dotpro_W2_18 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_18,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_18 );

dotpro_W2_19 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_19,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_19 );

dotpro_W2_20 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_20,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_20 );

dotpro_W2_21 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_21,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_21 );

dotpro_W2_22 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_22,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_22 );

dotpro_W2_23 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_23,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_23 );

dotpro_W2_24 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_24,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_24 );

dotpro_W2_25 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_25,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_25 );

dotpro_W2_26 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_26,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_26 );

dotpro_W2_27 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_27,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_27 );

dotpro_W2_28 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_28,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_28 );

dotpro_W2_29 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_29,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_29 );

dotpro_W2_30 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_30,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_30 );

dotpro_W2_31 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_31,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_31 );

dotpro_W2_32 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_32,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_32 );

dotpro_W2_33 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_33,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_33 );

dotpro_W2_34 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_34,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_34 );

dotpro_W2_35 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_35,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_35 );

dotpro_W2_36 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_36,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_36 );

dotpro_W2_37 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_37,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_37 );

dotpro_W2_38 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_38,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_38 );

dotpro_W2_39 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_39,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_39 );

dotpro_W2_40 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_40,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_40 );

dotpro_W2_41 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_41,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_41 );

dotpro_W2_42 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_42,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_42 );

dotpro_W2_43 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_43,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_43 );

dotpro_W2_44 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_44,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_44 );

dotpro_W2_45 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_45,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_45 );

dotpro_W2_46 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_46,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_46 );

dotpro_W2_47 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_47,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_47 );

dotpro_W2_48 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_48,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_48 );

dotpro_W2_49 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_49,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_49 );

dotpro_W2_50 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_50,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_50 );

dotpro_W2_51 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_51,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_51 );

dotpro_W2_52 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_52,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_52 );

dotpro_W2_53 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_53,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_53 );

dotpro_W2_54 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_54,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_54 );

dotpro_W2_55 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_55,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_55 );

dotpro_W2_56 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_56,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_56 );

dotpro_W2_57 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_57,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_57 );

dotpro_W2_58 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_58,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_58 );

dotpro_W2_59 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_59,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_59 );

dotpro_W2_60 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_60,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_60 );

dotpro_W2_61 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_61,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_61 );

dotpro_W2_62 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_62,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_62 );

dotpro_W2_63 : dotpro_W2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_63,
  CO9is2 => CO9is2,
  CO64is63 => CO64is63,
  MATMULOUT => mac_63 );


--CL2_63_inst : CL2_63 port map (
--  CLK => CLK,
--  RST_N => RST_N,
--  CO9 => CO9,
--  CO64 => CO64,
--  PixRomData => PixRomData,
--  W2Addr => W2Addr,
--  Z_WR_DATA => mac_63 );

---------------------------------------------------------
  truncate_00 <= mac_00(12 downto 6);
  truncate_01 <= mac_01(12 downto 6);
  truncate_02 <= mac_02(12 downto 6);
  truncate_03 <= mac_03(12 downto 6);
  truncate_04 <= mac_04(12 downto 6);
  truncate_05 <= mac_05(12 downto 6);
  truncate_06 <= mac_06(12 downto 6);
  truncate_07 <= mac_07(12 downto 6);
  truncate_08 <= mac_08(12 downto 6);
  truncate_09 <= mac_09(12 downto 6);
  truncate_10 <= mac_10(12 downto 6);
  truncate_11 <= mac_11(12 downto 6);
  truncate_12 <= mac_12(12 downto 6);
  truncate_13 <= mac_13(12 downto 6);
  truncate_14 <= mac_14(12 downto 6);
  truncate_15 <= mac_15(12 downto 6);
  truncate_16 <= mac_16(12 downto 6);
  truncate_17 <= mac_17(12 downto 6);
  truncate_18 <= mac_18(12 downto 6);
  truncate_19 <= mac_19(12 downto 6);
  truncate_20 <= mac_20(12 downto 6);
  truncate_21 <= mac_21(12 downto 6);
  truncate_22 <= mac_22(12 downto 6);
  truncate_23 <= mac_23(12 downto 6);
  truncate_24 <= mac_24(12 downto 6);
  truncate_25 <= mac_25(12 downto 6);
  truncate_26 <= mac_26(12 downto 6);
  truncate_27 <= mac_27(12 downto 6);
  truncate_28 <= mac_28(12 downto 6);
  truncate_29 <= mac_29(12 downto 6);
  truncate_30 <= mac_30(12 downto 6);
  truncate_31 <= mac_31(12 downto 6);
  truncate_32 <= mac_32(12 downto 6);
  truncate_33 <= mac_33(12 downto 6);
  truncate_34 <= mac_34(12 downto 6);
  truncate_35 <= mac_35(12 downto 6);
  truncate_36 <= mac_36(12 downto 6);
  truncate_37 <= mac_37(12 downto 6);
  truncate_38 <= mac_38(12 downto 6);
  truncate_39 <= mac_39(12 downto 6);
  truncate_40 <= mac_40(12 downto 6);
  truncate_41 <= mac_41(12 downto 6);
  truncate_42 <= mac_42(12 downto 6);
  truncate_43 <= mac_43(12 downto 6);
  truncate_44 <= mac_44(12 downto 6);
  truncate_45 <= mac_45(12 downto 6);
  truncate_46 <= mac_46(12 downto 6);
  truncate_47 <= mac_47(12 downto 6);
  truncate_48 <= mac_48(12 downto 6);
  truncate_49 <= mac_49(12 downto 6);
  truncate_50 <= mac_50(12 downto 6);
  truncate_51 <= mac_51(12 downto 6);
  truncate_52 <= mac_52(12 downto 6);
  truncate_53 <= mac_53(12 downto 6);
  truncate_54 <= mac_54(12 downto 6);
  truncate_55 <= mac_55(12 downto 6);
  truncate_56 <= mac_56(12 downto 6);
  truncate_57 <= mac_57(12 downto 6);
  truncate_58 <= mac_58(12 downto 6);
  truncate_59 <= mac_59(12 downto 6);
  truncate_60 <= mac_60(12 downto 6);
  truncate_61 <= mac_61(12 downto 6);
  truncate_62 <= mac_62(12 downto 6);
  truncate_63 <= mac_63(12 downto 6);

---------------------------------------------------------
  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);

---------------------------------------------------------
  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);

  Relu_16 <= (others => '0') when truncate_16 < 0 else
             (others => '1') when truncate_16 > 31 else
             truncate_16(4 downto 0);

  Relu_17 <= (others => '0') when truncate_17 < 0 else
             (others => '1') when truncate_17 > 31 else
             truncate_17(4 downto 0);

  Relu_18 <= (others => '0') when truncate_18 < 0 else
             (others => '1') when truncate_18 > 31 else
             truncate_18(4 downto 0);

  Relu_19 <= (others => '0') when truncate_19 < 0 else
             (others => '1') when truncate_19 > 31 else
             truncate_19(4 downto 0);

---------------------------------------------------------
  Relu_20 <= (others => '0') when truncate_20 < 0 else
             (others => '1') when truncate_20 > 31 else
             truncate_20(4 downto 0);

  Relu_21 <= (others => '0') when truncate_21 < 0 else
             (others => '1') when truncate_21 > 31 else
             truncate_21(4 downto 0);

  Relu_22 <= (others => '0') when truncate_22 < 0 else
             (others => '1') when truncate_22 > 31 else
             truncate_22(4 downto 0);

  Relu_23 <= (others => '0') when truncate_23 < 0 else
             (others => '1') when truncate_23 > 31 else
             truncate_23(4 downto 0);

  Relu_24 <= (others => '0') when truncate_24 < 0 else
             (others => '1') when truncate_24 > 31 else
             truncate_24(4 downto 0);

  Relu_25 <= (others => '0') when truncate_25 < 0 else
             (others => '1') when truncate_25 > 31 else
             truncate_25(4 downto 0);

  Relu_26 <= (others => '0') when truncate_26 < 0 else
             (others => '1') when truncate_26 > 31 else
             truncate_26(4 downto 0);

  Relu_27 <= (others => '0') when truncate_27 < 0 else
             (others => '1') when truncate_27 > 31 else
             truncate_27(4 downto 0);

  Relu_28 <= (others => '0') when truncate_28 < 0 else
             (others => '1') when truncate_28 > 31 else
             truncate_28(4 downto 0);

  Relu_29 <= (others => '0') when truncate_29 < 0 else
             (others => '1') when truncate_29 > 31 else
             truncate_29(4 downto 0);

---------------------------------------------------------
  Relu_30 <= (others => '0') when truncate_30 < 0 else
             (others => '1') when truncate_30 > 31 else
             truncate_30(4 downto 0);

  Relu_31 <= (others => '0') when truncate_31 < 0 else
             (others => '1') when truncate_31 > 31 else
             truncate_31(4 downto 0);

  Relu_32 <= (others => '0') when truncate_32 < 0 else
             (others => '1') when truncate_32 > 31 else
             truncate_32(4 downto 0);

  Relu_33 <= (others => '0') when truncate_33 < 0 else
             (others => '1') when truncate_33 > 31 else
             truncate_33(4 downto 0);

  Relu_34 <= (others => '0') when truncate_34 < 0 else
             (others => '1') when truncate_34 > 31 else
             truncate_34(4 downto 0);

  Relu_35 <= (others => '0') when truncate_35 < 0 else
             (others => '1') when truncate_35 > 31 else
             truncate_35(4 downto 0);

  Relu_36 <= (others => '0') when truncate_36 < 0 else
             (others => '1') when truncate_36 > 31 else
             truncate_36(4 downto 0);

  Relu_37 <= (others => '0') when truncate_37 < 0 else
             (others => '1') when truncate_37 > 31 else
             truncate_37(4 downto 0);

  Relu_38 <= (others => '0') when truncate_38 < 0 else
             (others => '1') when truncate_38 > 31 else
             truncate_38(4 downto 0);

  Relu_39 <= (others => '0') when truncate_39 < 0 else
             (others => '1') when truncate_39 > 31 else
             truncate_39(4 downto 0);

---------------------------------------------------------
  Relu_40 <= (others => '0') when truncate_40 < 0 else
             (others => '1') when truncate_40 > 31 else
             truncate_40(4 downto 0);

  Relu_41 <= (others => '0') when truncate_41 < 0 else
             (others => '1') when truncate_41 > 31 else
             truncate_41(4 downto 0);

  Relu_42 <= (others => '0') when truncate_42 < 0 else
             (others => '1') when truncate_42 > 31 else
             truncate_42(4 downto 0);

  Relu_43 <= (others => '0') when truncate_43 < 0 else
             (others => '1') when truncate_43 > 31 else
             truncate_43(4 downto 0);

  Relu_44 <= (others => '0') when truncate_44 < 0 else
             (others => '1') when truncate_44 > 31 else
             truncate_44(4 downto 0);

  Relu_45 <= (others => '0') when truncate_45 < 0 else
             (others => '1') when truncate_45 > 31 else
             truncate_45(4 downto 0);

  Relu_46 <= (others => '0') when truncate_46 < 0 else
             (others => '1') when truncate_46 > 31 else
             truncate_46(4 downto 0);

  Relu_47 <= (others => '0') when truncate_47 < 0 else
             (others => '1') when truncate_47 > 31 else
             truncate_47(4 downto 0);

  Relu_48 <= (others => '0') when truncate_48 < 0 else
             (others => '1') when truncate_48 > 31 else
             truncate_48(4 downto 0);

  Relu_49 <= (others => '0') when truncate_49 < 0 else
             (others => '1') when truncate_49 > 31 else
             truncate_49(4 downto 0);

---------------------------------------------------------
  Relu_50 <= (others => '0') when truncate_50 < 0 else
             (others => '1') when truncate_50 > 31 else
             truncate_50(4 downto 0);

  Relu_51 <= (others => '0') when truncate_51 < 0 else
             (others => '1') when truncate_51 > 31 else
             truncate_51(4 downto 0);

  Relu_52 <= (others => '0') when truncate_52 < 0 else
             (others => '1') when truncate_52 > 31 else
             truncate_52(4 downto 0);

  Relu_53 <= (others => '0') when truncate_53 < 0 else
             (others => '1') when truncate_53 > 31 else
             truncate_53(4 downto 0);

  Relu_54 <= (others => '0') when truncate_54 < 0 else
             (others => '1') when truncate_54 > 31 else
             truncate_54(4 downto 0);

  Relu_55 <= (others => '0') when truncate_55 < 0 else
             (others => '1') when truncate_55 > 31 else
             truncate_55(4 downto 0);

  Relu_56 <= (others => '0') when truncate_56 < 0 else
             (others => '1') when truncate_56 > 31 else
             truncate_56(4 downto 0);

  Relu_57 <= (others => '0') when truncate_57 < 0 else
             (others => '1') when truncate_57 > 31 else
             truncate_57(4 downto 0);

  Relu_58 <= (others => '0') when truncate_58 < 0 else
             (others => '1') when truncate_58 > 31 else
             truncate_58(4 downto 0);

  Relu_59 <= (others => '0') when truncate_59 < 0 else
             (others => '1') when truncate_59 > 31 else
             truncate_59(4 downto 0);

---------------------------------------------------------
  Relu_60 <= (others => '0') when truncate_60 < 0 else
             (others => '1') when truncate_60 > 31 else
             truncate_60(4 downto 0);

  Relu_61 <= (others => '0') when truncate_61 < 0 else
             (others => '1') when truncate_61 > 31 else
             truncate_61(4 downto 0);

  Relu_62 <= (others => '0') when truncate_62 < 0 else
             (others => '1') when truncate_62 > 31 else
             truncate_62(4 downto 0);

  Relu_63 <= (others => '0') when truncate_63 < 0 else
             (others => '1') when truncate_63 > 31 else
             truncate_63(4 downto 0);

ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)

		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)

		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)

		+ ("0000000" & Relu_30) + ("0000000" & Relu_31)
		+ ("0000000" & Relu_32) + ("0000000" & Relu_33)
		+ ("0000000" & Relu_34) + ("0000000" & Relu_35)
		+ ("0000000" & Relu_36) + ("0000000" & Relu_37)
		+ ("0000000" & Relu_38) + ("0000000" & Relu_39)

		+ ("0000000" & Relu_40) + ("0000000" & Relu_41)
		+ ("0000000" & Relu_42) + ("0000000" & Relu_43)
		+ ("0000000" & Relu_44) + ("0000000" & Relu_45)
		+ ("0000000" & Relu_46) + ("0000000" & Relu_47)
		+ ("0000000" & Relu_48) + ("0000000" & Relu_49)

		+ ("0000000" & Relu_50) + ("0000000" & Relu_51)
		+ ("0000000" & Relu_52) + ("0000000" & Relu_53)
		+ ("0000000" & Relu_54) + ("0000000" & Relu_55)
		+ ("0000000" & Relu_56) + ("0000000" & Relu_57)
		+ ("0000000" & Relu_58) + ("0000000" & Relu_59)

		+ ("0000000" & Relu_60) + ("0000000" & Relu_61)
		+ ("0000000" & Relu_62) + ("0000000" & Relu_63);

ReluAll <= Relu_00 & Relu_01
		& Relu_02 & Relu_03
		& Relu_04 & Relu_05
		& Relu_06 & Relu_07
		& Relu_08 & Relu_09

		& Relu_10 & Relu_11
		& Relu_12 & Relu_13
		& Relu_14 & Relu_15
		& Relu_16 & Relu_17
		& Relu_18 & Relu_19

		& Relu_20 & Relu_21
		& Relu_22 & Relu_23
		& Relu_24 & Relu_25
		& Relu_26 & Relu_27
		& Relu_28 & Relu_29

		& Relu_30 & Relu_31
		& Relu_32 & Relu_33
		& Relu_34 & Relu_35
		& Relu_36 & Relu_37
		& Relu_38 & Relu_39

		& Relu_40 & Relu_41
		& Relu_42 & Relu_43
		& Relu_44 & Relu_45
		& Relu_46 & Relu_47
		& Relu_48 & Relu_49

		& Relu_50 & Relu_51
		& Relu_52 & Relu_53
		& Relu_54 & Relu_55
		& Relu_56 & Relu_57
		& Relu_58 & Relu_59

		& Relu_60 & Relu_61
		& Relu_62 & Relu_63;

end RTL;

