-- A module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2020/02/23

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity pooling_lay2 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(319 downto 0); 
  WR_N : in std_logic; 
  HCO300 : in std_logic_vector(8 downto 0); 
  Poolout : Out std_logic_vector(319 downto 0) );  
end pooling_lay2;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of pooling_lay2 is

component SRAM_512x320
Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(8 downto 0);
          RDADDR : In std_logic_vector(8 downto 0);
          WRDATA : In std_logic_vector(319 downto 0);
          RDDATA : Out std_logic_vector(319 downto 0) );
end component;

component pooling_max
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(4 downto 0); 
  Relu1 : in std_logic_vector(4 downto 0); 
  WR_N : in std_logic; 
  HCO300 : in std_logic_vector(8 downto 0); 
  Poolout : Out std_logic_vector(4 downto 0) ); 
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal befdly_00,befdly_01,befdly_02,befdly_03,befdly_04,befdly_05,befdly_06,befdly_07,befdly_08,befdly_09 : std_logic_vector(4 downto 0);
signal befdly_10,befdly_11,befdly_12,befdly_13,befdly_14,befdly_15,befdly_16,befdly_17,befdly_18,befdly_19 : std_logic_vector(4 downto 0);
signal befdly_20,befdly_21,befdly_22,befdly_23,befdly_24,befdly_25,befdly_26,befdly_27,befdly_28,befdly_29 : std_logic_vector(4 downto 0);
signal befdly_30,befdly_31,befdly_32,befdly_33,befdly_34,befdly_35,befdly_36,befdly_37,befdly_38,befdly_39 : std_logic_vector(4 downto 0);
signal befdly_40,befdly_41,befdly_42,befdly_43,befdly_44,befdly_45,befdly_46,befdly_47,befdly_48,befdly_49 : std_logic_vector(4 downto 0);
signal befdly_50,befdly_51,befdly_52,befdly_53,befdly_54,befdly_55,befdly_56,befdly_57,befdly_58,befdly_59 : std_logic_vector(4 downto 0);
signal befdly_60,befdly_61,befdly_62,befdly_63 : std_logic_vector(4 downto 0);
signal aftdly_00,aftdly_01,aftdly_02,aftdly_03,aftdly_04,aftdly_05,aftdly_06,aftdly_07,aftdly_08,aftdly_09 : std_logic_vector(4 downto 0);
signal aftdly_10,aftdly_11,aftdly_12,aftdly_13,aftdly_14,aftdly_15,aftdly_16,aftdly_17,aftdly_18,aftdly_19 : std_logic_vector(4 downto 0);
signal aftdly_20,aftdly_21,aftdly_22,aftdly_23,aftdly_24,aftdly_25,aftdly_26,aftdly_27,aftdly_28,aftdly_29 : std_logic_vector(4 downto 0);
signal aftdly_30,aftdly_31,aftdly_32,aftdly_33,aftdly_34,aftdly_35,aftdly_36,aftdly_37,aftdly_38,aftdly_39 : std_logic_vector(4 downto 0);
signal aftdly_40,aftdly_41,aftdly_42,aftdly_43,aftdly_44,aftdly_45,aftdly_46,aftdly_47,aftdly_48,aftdly_49 : std_logic_vector(4 downto 0);
signal aftdly_50,aftdly_51,aftdly_52,aftdly_53,aftdly_54,aftdly_55,aftdly_56,aftdly_57,aftdly_58,aftdly_59 : std_logic_vector(4 downto 0);
signal aftdly_60,aftdly_61,aftdly_62,aftdly_63 : std_logic_vector(4 downto 0);
signal pool_00,pool_01,pool_02,pool_03,pool_04,pool_05,pool_06,pool_07,pool_08,pool_09 : std_logic_vector(4 downto 0);
signal pool_10,pool_11,pool_12,pool_13,pool_14,pool_15,pool_16,pool_17,pool_18,pool_19 : std_logic_vector(4 downto 0);
signal pool_20,pool_21,pool_22,pool_23,pool_24,pool_25,pool_26,pool_27,pool_28,pool_29 : std_logic_vector(4 downto 0);
signal pool_30,pool_31,pool_32,pool_33,pool_34,pool_35,pool_36,pool_37,pool_38,pool_39 : std_logic_vector(4 downto 0);
signal pool_40,pool_41,pool_42,pool_43,pool_44,pool_45,pool_46,pool_47,pool_48,pool_49 : std_logic_vector(4 downto 0);
signal pool_50,pool_51,pool_52,pool_53,pool_54,pool_55,pool_56,pool_57,pool_58,pool_59 : std_logic_vector(4 downto 0);
signal pool_60,pool_61,pool_62,pool_63 : std_logic_vector(4 downto 0);
signal Relu1 : std_logic_vector(319 downto 0);

begin
---------------------------------------------------------
RAM_P2_0 : SRAM_512x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> WR_N, 
   WRADDR	=> HCO300, 
   RDADDR	=> HCO300, 
   WRDATA	=> Relu0, 
   RDDATA	=> Relu1);

---------------------------------------------------------
pooling_max_00 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_00, 
   Relu1	=> aftdly_00, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_00);

pooling_max_01 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_01, 
   Relu1	=> aftdly_01, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_01);

pooling_max_02 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_02, 
   Relu1	=> aftdly_02, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_02);

pooling_max_03 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_03, 
   Relu1	=> aftdly_03, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_03);

pooling_max_04 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_04, 
   Relu1	=> aftdly_04, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_04);

pooling_max_05 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_05, 
   Relu1	=> aftdly_05, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_05);

pooling_max_06 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_06, 
   Relu1	=> aftdly_06, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_06);

pooling_max_07 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_07, 
   Relu1	=> aftdly_07, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_07);

pooling_max_08 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_08, 
   Relu1	=> aftdly_08, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_08);

pooling_max_09 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_09, 
   Relu1	=> aftdly_09, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_09);

---------------------------------------------------------
pooling_max_10 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_10, 
   Relu1	=> aftdly_10, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_10);

pooling_max_11 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_11, 
   Relu1	=> aftdly_11, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_11);

pooling_max_12 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_12, 
   Relu1	=> aftdly_12, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_12);

pooling_max_13 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_13, 
   Relu1	=> aftdly_13, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_13);

pooling_max_14 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_14, 
   Relu1	=> aftdly_14, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_14);

pooling_max_15 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_15, 
   Relu1	=> aftdly_15, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_15);

pooling_max_16 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_16, 
   Relu1	=> aftdly_16, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_16);

pooling_max_17 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_17, 
   Relu1	=> aftdly_17, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_17);

pooling_max_18 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_18, 
   Relu1	=> aftdly_18, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_18);

pooling_max_19 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_19, 
   Relu1	=> aftdly_19, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_19);

---------------------------------------------------------
pooling_max_20 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_20, 
   Relu1	=> aftdly_20, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_20);

pooling_max_21 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_21, 
   Relu1	=> aftdly_21, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_21);

pooling_max_22 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_22, 
   Relu1	=> aftdly_22, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_22);

pooling_max_23 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_23, 
   Relu1	=> aftdly_23, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_23);

pooling_max_24 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_24, 
   Relu1	=> aftdly_24, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_24);

pooling_max_25 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_25, 
   Relu1	=> aftdly_25, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_25);

pooling_max_26 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_26, 
   Relu1	=> aftdly_26, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_26);

pooling_max_27 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_27, 
   Relu1	=> aftdly_27, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_27);

pooling_max_28 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_28, 
   Relu1	=> aftdly_28, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_28);

pooling_max_29 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_29, 
   Relu1	=> aftdly_29, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_29);

---------------------------------------------------------
pooling_max_30 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_30, 
   Relu1	=> aftdly_30, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_30);

pooling_max_31 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_31, 
   Relu1	=> aftdly_31, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_31);

pooling_max_32 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_32, 
   Relu1	=> aftdly_32, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_32);

pooling_max_33 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_33, 
   Relu1	=> aftdly_33, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_33);

pooling_max_34 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_34, 
   Relu1	=> aftdly_34, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_34);

pooling_max_35 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_35, 
   Relu1	=> aftdly_35, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_35);

pooling_max_36 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_36, 
   Relu1	=> aftdly_36, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_36);

pooling_max_37 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_37, 
   Relu1	=> aftdly_37, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_37);

pooling_max_38 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_38, 
   Relu1	=> aftdly_38, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_38);

pooling_max_39 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_39, 
   Relu1	=> aftdly_39, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_39);

---------------------------------------------------------
pooling_max_40 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_40, 
   Relu1	=> aftdly_40, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_40);

pooling_max_41 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_41, 
   Relu1	=> aftdly_41, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_41);

pooling_max_42 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_42, 
   Relu1	=> aftdly_42, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_42);

pooling_max_43 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_43, 
   Relu1	=> aftdly_43, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_43);

pooling_max_44 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_44, 
   Relu1	=> aftdly_44, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_44);

pooling_max_45 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_45, 
   Relu1	=> aftdly_45, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_45);

pooling_max_46 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_46, 
   Relu1	=> aftdly_46, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_46);

pooling_max_47 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_47, 
   Relu1	=> aftdly_47, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_47);

pooling_max_48 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_48, 
   Relu1	=> aftdly_48, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_48);

pooling_max_49 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_49, 
   Relu1	=> aftdly_49, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_49);

---------------------------------------------------------
pooling_max_50 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_50, 
   Relu1	=> aftdly_50, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_50);

pooling_max_51 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_51, 
   Relu1	=> aftdly_51, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_51);

pooling_max_52 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_52, 
   Relu1	=> aftdly_52, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_52);

pooling_max_53 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_53, 
   Relu1	=> aftdly_53, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_53);

pooling_max_54 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_54, 
   Relu1	=> aftdly_54, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_54);

pooling_max_55 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_55, 
   Relu1	=> aftdly_55, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_55);

pooling_max_56 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_56, 
   Relu1	=> aftdly_56, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_56);

pooling_max_57 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_57, 
   Relu1	=> aftdly_57, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_57);

pooling_max_58 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_58, 
   Relu1	=> aftdly_58, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_58);

pooling_max_59 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_59, 
   Relu1	=> aftdly_59, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_59);

---------------------------------------------------------
pooling_max_60 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_60, 
   Relu1	=> aftdly_60, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_60);

pooling_max_61 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_61, 
   Relu1	=> aftdly_61, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_61);

pooling_max_62 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_62, 
   Relu1	=> aftdly_62, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_62);

pooling_max_63 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_63, 
   Relu1	=> aftdly_63, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_63);

befdly_00	<=	Relu0 (	319	downto	315	);
befdly_01	<=	Relu0 (	314	downto	310	);
befdly_02	<=	Relu0 (	309	downto	305	);
befdly_03	<=	Relu0 (	304	downto	300	);
befdly_04	<=	Relu0 (	299	downto	295	);
befdly_05	<=	Relu0 (	294	downto	290	);
befdly_06	<=	Relu0 (	289	downto	285	);
befdly_07	<=	Relu0 (	284	downto	280	);
befdly_08	<=	Relu0 (	279	downto	275	);
befdly_09	<=	Relu0 (	274	downto	270	);
befdly_10	<=	Relu0 (	269	downto	265	);
befdly_11	<=	Relu0 (	264	downto	260	);
befdly_12	<=	Relu0 (	259	downto	255	);
befdly_13	<=	Relu0 (	254	downto	250	);
befdly_14	<=	Relu0 (	249	downto	245	);
befdly_15	<=	Relu0 (	244	downto	240	);
befdly_16	<=	Relu0 (	239	downto	235	);
befdly_17	<=	Relu0 (	234	downto	230	);
befdly_18	<=	Relu0 (	229	downto	225	);
befdly_19	<=	Relu0 (	224	downto	220	);
befdly_20	<=	Relu0 (	219	downto	215	);
befdly_21	<=	Relu0 (	214	downto	210	);
befdly_22	<=	Relu0 (	209	downto	205	);
befdly_23	<=	Relu0 (	204	downto	200	);
befdly_24	<=	Relu0 (	199	downto	195	);
befdly_25	<=	Relu0 (	194	downto	190	);
befdly_26	<=	Relu0 (	189	downto	185	);
befdly_27	<=	Relu0 (	184	downto	180	);
befdly_28	<=	Relu0 (	179	downto	175	);
befdly_29	<=	Relu0 (	174	downto	170	);
befdly_30	<=	Relu0 (	169	downto	165	);
befdly_31	<=	Relu0 (	164	downto	160	);
befdly_32	<=	Relu0 (	159	downto	155	);
befdly_33	<=	Relu0 (	154	downto	150	);
befdly_34	<=	Relu0 (	149	downto	145	);
befdly_35	<=	Relu0 (	144	downto	140	);
befdly_36	<=	Relu0 (	139	downto	135	);
befdly_37	<=	Relu0 (	134	downto	130	);
befdly_38	<=	Relu0 (	129	downto	125	);
befdly_39	<=	Relu0 (	124	downto	120	);
befdly_40	<=	Relu0 (	119	downto	115	);
befdly_41	<=	Relu0 (	114	downto	110	);
befdly_42	<=	Relu0 (	109	downto	105	);
befdly_43	<=	Relu0 (	104	downto	100	);
befdly_44	<=	Relu0 (	99	downto	95	);
befdly_45	<=	Relu0 (	94	downto	90	);
befdly_46	<=	Relu0 (	89	downto	85	);
befdly_47	<=	Relu0 (	84	downto	80	);
befdly_48	<=	Relu0 (	79	downto	75	);
befdly_49	<=	Relu0 (	74	downto	70	);
befdly_50	<=	Relu0 (	69	downto	65	);
befdly_51	<=	Relu0 (	64	downto	60	);
befdly_52	<=	Relu0 (	59	downto	55	);
befdly_53	<=	Relu0 (	54	downto	50	);
befdly_54	<=	Relu0 (	49	downto	45	);
befdly_55	<=	Relu0 (	44	downto	40	);
befdly_56	<=	Relu0 (	39	downto	35	);
befdly_57	<=	Relu0 (	34	downto	30	);
befdly_58	<=	Relu0 (	29	downto	25	);
befdly_59	<=	Relu0 (	24	downto	20	);
befdly_60	<=	Relu0 (	19	downto	15	);
befdly_61	<=	Relu0 (	14	downto	10	);
befdly_62	<=	Relu0 (	9	downto	5	);
befdly_63	<=	Relu0 (	4	downto	0	);

aftdly_00	<=	Relu1 (	319	downto	315	);
aftdly_01	<=	Relu1 (	314	downto	310	);
aftdly_02	<=	Relu1 (	309	downto	305	);
aftdly_03	<=	Relu1 (	304	downto	300	);
aftdly_04	<=	Relu1 (	299	downto	295	);
aftdly_05	<=	Relu1 (	294	downto	290	);
aftdly_06	<=	Relu1 (	289	downto	285	);
aftdly_07	<=	Relu1 (	284	downto	280	);
aftdly_08	<=	Relu1 (	279	downto	275	);
aftdly_09	<=	Relu1 (	274	downto	270	);
aftdly_10	<=	Relu1 (	269	downto	265	);
aftdly_11	<=	Relu1 (	264	downto	260	);
aftdly_12	<=	Relu1 (	259	downto	255	);
aftdly_13	<=	Relu1 (	254	downto	250	);
aftdly_14	<=	Relu1 (	249	downto	245	);
aftdly_15	<=	Relu1 (	244	downto	240	);
aftdly_16	<=	Relu1 (	239	downto	235	);
aftdly_17	<=	Relu1 (	234	downto	230	);
aftdly_18	<=	Relu1 (	229	downto	225	);
aftdly_19	<=	Relu1 (	224	downto	220	);
aftdly_20	<=	Relu1 (	219	downto	215	);
aftdly_21	<=	Relu1 (	214	downto	210	);
aftdly_22	<=	Relu1 (	209	downto	205	);
aftdly_23	<=	Relu1 (	204	downto	200	);
aftdly_24	<=	Relu1 (	199	downto	195	);
aftdly_25	<=	Relu1 (	194	downto	190	);
aftdly_26	<=	Relu1 (	189	downto	185	);
aftdly_27	<=	Relu1 (	184	downto	180	);
aftdly_28	<=	Relu1 (	179	downto	175	);
aftdly_29	<=	Relu1 (	174	downto	170	);
aftdly_30	<=	Relu1 (	169	downto	165	);
aftdly_31	<=	Relu1 (	164	downto	160	);
aftdly_32	<=	Relu1 (	159	downto	155	);
aftdly_33	<=	Relu1 (	154	downto	150	);
aftdly_34	<=	Relu1 (	149	downto	145	);
aftdly_35	<=	Relu1 (	144	downto	140	);
aftdly_36	<=	Relu1 (	139	downto	135	);
aftdly_37	<=	Relu1 (	134	downto	130	);
aftdly_38	<=	Relu1 (	129	downto	125	);
aftdly_39	<=	Relu1 (	124	downto	120	);
aftdly_40	<=	Relu1 (	119	downto	115	);
aftdly_41	<=	Relu1 (	114	downto	110	);
aftdly_42	<=	Relu1 (	109	downto	105	);
aftdly_43	<=	Relu1 (	104	downto	100	);
aftdly_44	<=	Relu1 (	99	downto	95	);
aftdly_45	<=	Relu1 (	94	downto	90	);
aftdly_46	<=	Relu1 (	89	downto	85	);
aftdly_47	<=	Relu1 (	84	downto	80	);
aftdly_48	<=	Relu1 (	79	downto	75	);
aftdly_49	<=	Relu1 (	74	downto	70	);
aftdly_50	<=	Relu1 (	69	downto	65	);
aftdly_51	<=	Relu1 (	64	downto	60	);
aftdly_52	<=	Relu1 (	59	downto	55	);
aftdly_53	<=	Relu1 (	54	downto	50	);
aftdly_54	<=	Relu1 (	49	downto	45	);
aftdly_55	<=	Relu1 (	44	downto	40	);
aftdly_56	<=	Relu1 (	39	downto	35	);
aftdly_57	<=	Relu1 (	34	downto	30	);
aftdly_58	<=	Relu1 (	29	downto	25	);
aftdly_59	<=	Relu1 (	24	downto	20	);
aftdly_60	<=	Relu1 (	19	downto	15	);
aftdly_61	<=	Relu1 (	14	downto	10	);
aftdly_62	<=	Relu1 (	9	downto	5	);
aftdly_63	<=	Relu1 (	4	downto	0	);

Poolout <= pool_00 & pool_01
		& pool_02 & pool_03
		& pool_04 & pool_05
		& pool_06 & pool_07
		& pool_08 & pool_09

		& pool_10 & pool_11
		& pool_12 & pool_13
		& pool_14 & pool_15
		& pool_16 & pool_17
		& pool_18 & pool_19

		& pool_20 & pool_21
		& pool_22 & pool_23
		& pool_24 & pool_25
		& pool_26 & pool_27
		& pool_28 & pool_29

		& pool_30 & pool_31
		& pool_32 & pool_33
		& pool_34 & pool_35
		& pool_36 & pool_37
		& pool_38 & pool_39

		& pool_40 & pool_41
		& pool_42 & pool_43
		& pool_44 & pool_45
		& pool_46 & pool_47
		& pool_48 & pool_49

		& pool_50 & pool_51
		& pool_52 & pool_53
		& pool_54 & pool_55
		& pool_56 & pool_57
		& pool_58 & pool_59

		& pool_60 & pool_61
		& pool_62 & pool_63;

end RTL;

