-- The top module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2019/02/13

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity ssd_hw is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  WR_BUF_SEL : in std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : in std_logic_vector(1 downto 0);
  WR_BUF_N : in std_logic;
  RD_BUF_SEL : in std_logic_vector(1 downto 0);
  RD_BUF_ADDR : in std_logic_vector(8 downto 0);
  SsdLineCount : in std_logic_vector(8 downto 0);
  SsdLineCount2 : in std_logic_vector(8 downto 0);
  SsdPixCount : in std_logic_vector(8 downto 0);
  SsdPixCount2 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO64 : in std_logic_vector(5 downto 0);
  CO64_dly2 : in std_logic_vector(5 downto 0);
--		BigLineN : in std_logic_vector(8 downto 0);
		ssd_RGB : in std_logic_vector(15 downto 0);
		DISPJUDGE : in std_logic;
  ReluWrN : out std_logic;
  ReluWrData : out std_logic_vector(319 downto 0);
  ReluWrAddr : out std_logic_vector(8 downto 0);
  ReluWrSel : out std_logic_vector(1 downto 0);
  Lay2WrN : in std_logic;
  Lay2WrData : in std_logic_vector(319 downto 0);
  Lay2WrAddr : in std_logic_vector(8 downto 0);
  Lay2WrSel : in std_logic_vector(1 downto 0);
		ReluSum : out std_logic_vector(11 downto 0) );
end ssd_hw;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of ssd_hw is

component conv_layer2
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO64 : in std_logic_vector(5 downto 0);
  CO64_dly2 : in std_logic_vector(5 downto 0);
  X_RD_DATA : in std_logic_vector(319 downto 0); 
  ReluAll : out std_logic_vector(319 downto 0); 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end component;

component SRAM_512x320
Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(8 downto 0);
          RDADDR : In std_logic_vector(8 downto 0);
          WRDATA : In std_logic_vector(319 downto 0);
          RDDATA : Out std_logic_vector(319 downto 0) );
end component;

component SRAM_256x320
Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(7 downto 0);
          RDADDR : In std_logic_vector(7 downto 0);
          WRDATA : In std_logic_vector(319 downto 0);
          RDDATA : Out std_logic_vector(319 downto 0) );
end component;

component pooling_lay2
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(319 downto 0); 
  WR_N : in std_logic; 
  HCO300 : in std_logic_vector(8 downto 0); 
  Poolout : Out std_logic_vector(319 downto 0) );  
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal BchData : std_logic_vector(6 downto 0);
signal GchData : std_logic_vector(6 downto 0);
signal RchData : std_logic_vector(6 downto 0);
signal Bch_oft, Bch : std_logic_vector(5 downto 0);
signal Gch_oft, Gch : std_logic_vector(6 downto 0);
signal Rch_oft, Rch : std_logic_vector(5 downto 0);

signal RAM_X1_WR_N_0, RAM_X1_WR_N_1, RAM_X1_WR_N_2, RAM_X1_WR_N_3 : std_logic;
signal RAM_X1_WR_ADDR : std_logic_vector(8 downto 0);
signal RAM_X1_RD_ADDR : std_logic_vector(8 downto 0);
signal RAM_X1_WR_DATA : std_logic_vector(319 downto 0);
signal RAM_X1_RD_DATA_0, RAM_X1_RD_DATA_1, RAM_X1_RD_DATA_2, RAM_X1_RD_DATA_3 : std_logic_vector(319 downto 0);
signal RAM_X1_RD_DATA_1_padding, RAM_X1_RD_DATA_2_padding : std_logic_vector(319 downto 0);
signal RAM_X1_RD_DATA : std_logic_vector(319 downto 0);
signal RAM_X1_RD_SEL : std_logic_vector(1 downto 0);

signal RAM_X2_WR_N_0, RAM_X2_WR_N_1, RAM_X2_WR_N_2, RAM_X2_WR_N_3 : std_logic;
signal RAM_X2_WR_ADDR : std_logic_vector(7 downto 0);
signal RAM_X2_RD_ADDR : std_logic_vector(7 downto 0);
signal RAM_X2_WR_DATA : std_logic_vector(319 downto 0);
signal RAM_X2_RD_DATA_0, RAM_X2_RD_DATA_1, RAM_X2_RD_DATA_2, RAM_X2_RD_DATA_3 : std_logic_vector(319 downto 0);
signal RAM_X2_RD_DATA : std_logic_vector(319 downto 0);

signal ReluAll : std_logic_vector(319 downto 0);
signal ReluWrN_sig : std_logic;
signal HCO300 : std_logic_vector(8 downto 0);
signal Poolout : std_logic_vector(319 downto 0);
signal Pooladdr : std_logic_vector(7 downto 0);
signal SsdLineCount2_sig : std_logic_vector(9 downto 0);

begin

---------------------------------------------------------
  Bch_oft <= '0' & ssd_RGB(4 downto 0);
  Bch <= (Bch_oft - 16);
  BchData <= Bch & "0"; 

  Gch_oft <= '0' & ssd_RGB(10 downto 5);
  Gch <= (Gch_oft - 32);
  GchData <= Gch; 

  Rch_oft <= '0' & ssd_RGB(15 downto 11);
  Rch <= (Rch_oft - 16);
  RchData <= Rch & "0"; 

---------------------------------------------------------
  SsdLineCount2_sig <= '0' & SsdLineCount2; -- put a sign bit

  ReluWrN_sig <= '0' when WR_BUF_N = '0' 
	and (SsdLineCount2_sig >= 2 and SsdLineCount2_sig <= 301) 
	and SsdLineCount2_sig(0) = '1' and SsdPixCount2(0) = '1' else '1'; -- layer 2 has poolings

  ReluWrSel <= WR_BUF_SEL2;
  ReluWrN <= ReluWrN_sig;
  ReluWrData <= RAM_X2_WR_DATA;
  ReluWrAddr <= '0' & RAM_X2_WR_ADDR;

---------------------------------------------------------
RAM_X1_0 : SRAM_512x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_0, 
   WRADDR	=> Lay2WrAddr, -- RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> Lay2WrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_0);

-- RAM_X1_WR_N_0 <= '0' when WR_BUF_N = '0' and WR_BUF_SEL = "00" else '1';
RAM_X1_WR_N_0 <= '0' when Lay2WrN = '0' and Lay2WrSel = "00" else '1';

---------------------------------------------------------
RAM_X1_1 : SRAM_512x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_1, 
   WRADDR	=> Lay2WrAddr, -- RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> Lay2WrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_1);

-- RAM_X1_WR_N_1 <= '0' when WR_BUF_N = '0' and WR_BUF_SEL = "01" else '1';
RAM_X1_WR_N_1 <= '0' when Lay2WrN = '0' and Lay2WrSel = "01" else '1';

---------------------------------------------------------
RAM_X1_2 : SRAM_512x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_2, 
   WRADDR	=> Lay2WrAddr, -- RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> Lay2WrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_2);

-- RAM_X1_WR_N_2 <= '0' when WR_BUF_N = '0' and WR_BUF_SEL = "10" else '1';
RAM_X1_WR_N_2 <= '0' when Lay2WrN = '0' and Lay2WrSel = "10" else '1';

---------------------------------------------------------
RAM_X1_3 : SRAM_512x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_3, 
   WRADDR	=> Lay2WrAddr, -- RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> Lay2WrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_3);

-- RAM_X1_WR_N_3 <= '0' when WR_BUF_N = '0' and WR_BUF_SEL = "11" else '1';
RAM_X1_WR_N_3 <= '0' when Lay2WrN = '0' and Lay2WrSel = "11" else '1';

---------------------------------------------------------
RAM_X1_RD_ADDR <= RD_BUF_ADDR;

RAM_X1_RD_SEL <= RD_BUF_SEL - 2; -- because RAM_X1 is written from '2'

RAM_X1_RD_DATA <= RAM_X1_RD_DATA_0 when RAM_X1_RD_SEL = "00" else 
		RAM_X1_RD_DATA_1_padding when RAM_X1_RD_SEL = "01" else 
		RAM_X1_RD_DATA_2_padding when RAM_X1_RD_SEL = "10" else 
		RAM_X1_RD_DATA_3;

RAM_X1_RD_DATA_1_padding <= (others => '0') when SsdLineCount = 2 else RAM_X1_RD_DATA_1;

RAM_X1_RD_DATA_2_padding <= (others => '0') when SsdLineCount = 301 or SsdLineCount = 302 else RAM_X1_RD_DATA_2; -- ram_x1_rd_sel 0,1,2,0,1,2 -> 1,2,3,1,2,3 accumulator could be unknown without 302, co64is63 catches unknown 2022/04/30

conv_layer2_inst : conv_layer2 port map (
          CLK => CLK,
          RST_N => RST_N,
          CO300 => SsdLineCount,
          CO9 => CO9,
          CO64 => CO64,
          CO64_dly2 => CO64_dly2,
          X_RD_DATA => RAM_X1_RD_DATA,
          ReluAll => ReluAll,
          ReluSum => open);

--RAM_X2_WR_ADDR <= SsdPixCount;
RAM_X2_WR_ADDR <= Pooladdr; -- layer2 has poolings
--RAM_X2_WR_DATA <= ReluAll;
RAM_X2_WR_DATA <= Poolout; -- layer2 has poolings

---------------------------------------------------------
RAM_X2_0 : SRAM_256x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X2_WR_N_0, 
   WRADDR	=> RAM_X2_WR_ADDR, 
   RDADDR	=> RAM_X2_RD_ADDR, 
   WRDATA	=> RAM_X2_WR_DATA, 
   RDDATA	=> RAM_X2_RD_DATA_0);

RAM_X2_WR_N_0 <= '0' when ReluWrN_sig = '0' and WR_BUF_SEL2 = "00" else '1'; -- layer 2 has poolings

---------------------------------------------------------
RAM_X2_1 : SRAM_256x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X2_WR_N_1, 
   WRADDR	=> RAM_X2_WR_ADDR, 
   RDADDR	=> RAM_X2_RD_ADDR, 
   WRDATA	=> RAM_X2_WR_DATA, 
   RDDATA	=> RAM_X2_RD_DATA_1);

RAM_X2_WR_N_1 <= '0' when ReluWrN_sig = '0' and WR_BUF_SEL2 = "01" else '1'; -- layer 2 has poolings

---------------------------------------------------------
RAM_X2_2 : SRAM_256x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X2_WR_N_2, 
   WRADDR	=> RAM_X2_WR_ADDR, 
   RDADDR	=> RAM_X2_RD_ADDR, 
   WRDATA	=> RAM_X2_WR_DATA, 
   RDDATA	=> RAM_X2_RD_DATA_2);

RAM_X2_WR_N_2 <= '0' when ReluWrN_sig = '0' and WR_BUF_SEL2 = "10" else '1'; -- layer 2 has poolings

---------------------------------------------------------
RAM_X2_3 : SRAM_256x320 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X2_WR_N_3, 
   WRADDR	=> RAM_X2_WR_ADDR, 
   RDADDR	=> RAM_X2_RD_ADDR, 
   WRDATA	=> RAM_X2_WR_DATA, 
   RDDATA	=> RAM_X2_RD_DATA_3);

RAM_X2_WR_N_3 <= '0' when ReluWrN_sig = '0' and WR_BUF_SEL2 = "11" else '1'; -- layer 2 has poolings

---------------------------------------------------------
pooling_lay2_inst : pooling_lay2 port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> ReluAll,
   WR_N	=> WR_BUF_N,
   HCO300	=> HCO300,
   Poolout	=> Poolout);

HCO300 <= SsdPixCount - 0;

Pooladdr <= SsdPixCount2(8 downto 1) + 1;

end RTL;

